package gov.va.med.hac.edi.ewv2.actions;

import gov.va.med.hac.edi.ewv2.beans.claim.*;
import gov.va.med.hac.edi.ewv2.beans.demographics.*;
import gov.va.med.hac.edi.ewv2.beans.ohi.OhiInfoContainer;
import gov.va.med.hac.edi.ewv2.exceptions.*;
import gov.va.med.hac.edi.ewv2.persistence.EwvClaim;
import gov.va.med.hac.edi.ewv2.persistence.UserLog;
import gov.va.med.hac.edi.ewv2.persistence.base.*;
import gov.va.med.hac.edi.ewv2.utils.*;

import java.io.*;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

import javax.faces.application.FacesMessage;
import javax.faces.context.FacesContext;
import javax.persistence.*;

import org.jboss.seam.*;
import org.jboss.seam.annotations.*;
import org.jboss.seam.annotations.security.Restrict;
import org.jboss.seam.log.Log;

@Name("Ewv2AllClaimInfoAction")
@Scope(ScopeType.CONVERSATION)
public class AllClaimInfoAction implements Serializable {
  private static final long serialVersionUID = 5215329070049375734L;

  @In(create = true)
  protected EntityManager entityManager;
  
  @Logger
  protected Log log;
  
  @Out("Ewv2PatientHeaderInfo")
  private PatientHeaderInfo patientHeaderInfo;

  @Out("Ewv2CommonDemographicsInfo")
  private CommonDemographicsContainer commonDemographicsInfo;

  @Out("Ewv2ClaimService")
  private ClaimService claimService;

  @In("Ewv2PdiNumberBean")
  private PdiNumberBean pdiNumberBean;

  @Out("Ewv2ClaimContainer")
  private ClaimContainer claimContainer;

  @Out("Ewv2OhiInfoContainer")
  private OhiInfoContainer ohiInfoContainer;

  private boolean doDebug;
  
  @Restrict("#{s:hasRole('ewvUser')}")
  public void getAllClaimInfo() throws ActionErrorException, NonUniqueRecordsException {
    //log.info("getAllClaimInfo START");
    ClaimFormat claimFormat = null;
    ClaimType claimType = null;
    String pdiNumber = null;
    
    try {
      String unfilteredPdiNumber = pdiNumberBean.getPdiNumber();
      
      pdiNumber = filterPdiNumber(unfilteredPdiNumber);
    
      // Check the application properties file to see whether to carry out a test error.
      considerTestErrorAndDebug(pdiNumber);
      
      //log.info("getAllClaimInfo pdiNumber: " + pdiNumber);
      recordPdiNumber(pdiNumber);
  
      invalidate();
  
      //log.info("Claim Query Start");
      String errorMessage = "Unable to find submission for this PDI.";
      EwvClaim claimBean = getClaimRecord(pdiNumber);
  
      if (claimBean != null) {
      
        // Put together a Data Access Object for a few pieces common to both tabs
        //log.info("Demographics Start");
        commonDemographicsInfo = new DemographicsDataBinder().getCommonDemographics(claimBean);
        
        claimFormat = DemographicsDataBinder.determineClaimFormat(claimBean);
        claimType = DemographicsDataBinder.determineClaimType(claimBean);

        // The Patient Header appears near the top of the screen
        patientHeaderInfo = DemographicsDataBinder.populatePatientHeaderInfo(claimBean);

        //log.info("Claim Start");
        claimContainer = new ClaimDataBinder().populateClaimContainer(claimBean, log);
        
        // Most of the difficulties lie in the area of OHIs
        //log.info("OHI Start");
        ohiInfoContainer = new OhiDataBinder().getOhiData(claimBean, log);

        claimService = (ClaimService) Component.getInstance(ClaimService.class);
        if (claimService != null) {
          claimService.setClaimType(claimType);
          claimService.setClaimFormat(claimFormat);
          claimService.setIsClaimFound(true);
          claimService.setDebugOn(doDebug);
        }
        
        /* 
         * This section of code will handle the logging of end user's distinct PDI request for a given day (24 hour days). 
         * 03/22/2016 - Part of HAPE Project - EWV 837 Task - VL
         */
        
        String pdiTrackStatusMessage = trackPdiRequest(claimService.getUserName(), pdiNumber);
        // Verify the PDI request is unique for a given user and date
        
        if(pdiTrackStatusMessage != null){
        	String displayMessage = "PDI Queries " + pdiTrackStatusMessage;
        	FacesContext.getCurrentInstance().addMessage("ewvApplication",
                    new FacesMessage(FacesMessage.SEVERITY_ERROR, displayMessage, ""));
        }   
      } else {
        // No claim was found
        FacesContext.getCurrentInstance().addMessage("ewvApplication",
            new FacesMessage(FacesMessage.SEVERITY_ERROR, errorMessage, ""));
      }

    } catch (Throwable e) {
      log.info("DEBUG: Caught a throwable: " + e);
      handleException(pdiNumber, e);
    }

  }

  protected void recordPdiNumber(String number)
  {
    MostRecentPdiNumbers recents = (MostRecentPdiNumbers) Component.getInstance("Ewv2RecentPdiNumbers");
    if (recents != null) {
        //log.info("recording PDI number: " + number);
        recents.addPdi(number);
    }
    else {
        log.error("Cannot get Instance of MostRecentPdiNumbers cache");
    }
  }
  
  protected EwvClaim getClaimRecord(String pdiNumber) throws ActionErrorException, NonUniqueRecordsException
  {
      EwvClaim ret = null;
      String queryString ="select claim from EwvClaim as claim where claim.pdi = :pdi";
      try {
          Query query = entityManager.createQuery(queryString);
          query.setParameter("pdi", pdiNumber);
          ret = (EwvClaim) query.getSingleResult();
      } catch (Throwable t) {
          if (t instanceof javax.persistence.NoResultException)
          {
              log.info("cannot get record for PDI: " + pdiNumber);
          }
          else if(t instanceof javax.persistence.NonUniqueResultException)
          {
              log.error("more than 1 claim exists for PDI: "+pdiNumber);
              
              // this means that there are crap claim records in DB
              throw new NonUniqueRecordsException("more than 1 claim exists for PDI: "+pdiNumber);
          }
          else
          {
              // log exception and route user to error page
              handleException(pdiNumber, t);
          }
      }
      
      return ret;
  }
  
  protected String trackPdiRequest(String userName, String pdi) throws ActionErrorException, NonUniqueRecordsException
  {
	  DateFormat dateFormat = new SimpleDateFormat("yyyy.MM.dd");
	  Date currentDate = new Date();
	  String currentDateString = dateFormat.format(currentDate);
	  boolean uniquePdiRequest = true;
	  int pdiRequestCount = 0;
	  
	  uniquePdiRequest = isUniquePdiRequest(userName, pdi, currentDateString);

	  if (uniquePdiRequest) {
		  createUserLog(userName, pdi);
	  } 
	  
	  pdiRequestCount = getPdiRequestCount(userName, currentDateString);
	  
	  return Integer.toString(pdiRequestCount);
	  //return null;
  }

  protected boolean isUniquePdiRequest(String username, String pdiNumber, String requestDate) throws ActionErrorException, NonUniqueRecordsException
  {
      boolean ret = true;
      String queryString = null;
      Query query;
      
      try {
          // Determine if the user has any query for current date
          queryString = "select count(*) from ewv.user_log u where u.user_name = :username and to_char(u.request_date, 'yyyy.MM.dd') < :requestdate";
          
          query = entityManager.createNativeQuery(queryString);
          query.setParameter("username", username);
          query.setParameter("requestdate", requestDate);     
          BigDecimal recordCurrentDateCount = (BigDecimal) query.getSingleResult();
          
          // If not current date, delete all request for User. Else determine if it is a unique Pdi for the current date
          if (recordCurrentDateCount.intValue() > 0) {
              queryString = "delete from ewv.user_log u where u.user_name = :username";
              
              query = entityManager.createNativeQuery(queryString);
              query.setParameter("username", username);
              
              query.executeUpdate();
             	  
        	  return true;
          } else { 	  
              queryString ="select count(*) from ewv.user_log u where u.pdi = :pdi and u.user_name = :username and to_char(u.request_date, 'yyyy.MM.dd') = :requestdate";
              query = entityManager.createNativeQuery(queryString);
              query.setParameter("pdi", pdiNumber);
              query.setParameter("username", username);
              query.setParameter("requestdate", requestDate);     
              BigDecimal recordCount = (BigDecimal) query.getSingleResult();
              
              if (recordCount.intValue() > 0) {
            	  return false;
              }
          }
      } catch (Throwable t) {
          if (t instanceof javax.persistence.NoResultException)
          {
              log.info("cannot get record for PDI: " + pdiNumber);
          }
          else if(t instanceof javax.persistence.NonUniqueResultException)
          {
              log.error("more than 1 claim exists for PDI: "+pdiNumber);
              
              // this means that there are crap claim records in DB
              throw new NonUniqueRecordsException("more than 1 claim exists for PDI: "+pdiNumber);
          }
          else
          {
              // log exception and route user to error page
              handleException(pdiNumber, t);
          }
      }
      
      return ret;
  }
  
  protected void createUserLog(String userName, String pdi) throws ActionErrorException, NonUniqueRecordsException
  {
	  
	 // java.util.Date today = new java.util.Date();
	  String sql = null;
	  Query q;
  
      try {
          sql = " INSERT INTO ewv.user_log(user_name, " 
            		+ " pdi, " 
            		+ " request_date)" 
            		+ " VALUES("
  	           	+ ":username, "
  	           	+ ":pdi, " 
  	           	+ "sysdate) ";
        q = entityManager.createNativeQuery(sql);
        q.setParameter("username", userName);
        q.setParameter("pdi", pdi);

        q.executeUpdate();
      } catch (Throwable t) {
          if (t instanceof javax.persistence.NoResultException)
          {
              log.info("cannot get record for PDI: " + pdi);
          }
          else if(t instanceof javax.persistence.NonUniqueResultException)
          {
              log.error("more than 1 claim exists for PDI: "+pdi);
              
              // this means that there are crap claim records in DB
              throw new NonUniqueRecordsException("more than 1 claim exists for PDI: "+pdi);
          }
          else
          {
              // log exception and route user to error page
              handleException(pdi, t);
          }
      }
  }
  
  protected int getPdiRequestCount(String username, String requestDate) throws ActionErrorException, NonUniqueRecordsException
  {
      int ret = 0;

      String queryString ="select count(*) from ewv.user_log u where u.user_name = :username and to_char(u.request_date, 'yyyy.MM.dd') = :requestdate";
      try {
          Query query = entityManager.createNativeQuery(queryString);
          query.setParameter("username", username);
          query.setParameter("requestdate", requestDate);     
          BigDecimal recordCount = (BigDecimal) query.getSingleResult();
          
          if (recordCount.intValue() > 0) {
        	  return recordCount.intValue();
          }
          
      } catch (Throwable t) {
          if (t instanceof javax.persistence.NoResultException)
          {
              log.info("cannot get record for PDI: ");
          }
          else if(t instanceof javax.persistence.NonUniqueResultException)
          {
              log.error("more than 1 claim exists for PDI: ");
              
              // this means that there are crap claim records in DB
              throw new NonUniqueRecordsException("more than 1 claim exists for PDI: ");
          }
          else
          {
              // log exception and route user to error page
              handleException(username, t);
          }
      }
      
      return ret;
  }
  
  public String filterPdiNumber(String pdiNumber)
  {
    String returnVal = pdiNumber;
    
    if(returnVal != null) {
      returnVal = returnVal.replace(" ", "");
    }
    
    return returnVal;
  }
  
  public String filterPdiNumberDeprecatedButWorks(String pdiNumber)
  {
      CharSequence seq = new StringBuffer().append(" ").toString();
      
      if(pdiNumber != null && pdiNumber.contains(seq)) {
          pdiNumber = pdiNumber.replaceAll(" ", "");
      }
      
      return pdiNumber;
  }
  
  public void invalidate() {
    commonDemographicsInfo
        = new CommonDemographicsContainer(new PatientDataBean(),
          new ServiceLocationBean(),
          new VendorDataBean());

    // patient header panel
    patientHeaderInfo = new PatientHeaderInfo();

    // billing and ohi tab data...
    claimContainer = new ClaimContainer();
    ohiInfoContainer = new OhiInfoContainer();

    // the demographics tab data...
    PatientDataBean patientDataBean = new PatientDataBean();
    ServiceLocationBean serviceLocationDataBean = new ServiceLocationBean();
    VendorDataBean vendorDataBean = new VendorDataBean();

    vendorDataBean.setShouldRender(false);
    serviceLocationDataBean.setShouldRender(false);
    patientDataBean.setShouldRender(false);

    //log.info("PACKAGE START");

    commonDemographicsInfo
      = new CommonDemographicsContainer(patientDataBean,
                                        serviceLocationDataBean,
                                        vendorDataBean);

    claimService = (ClaimService) Component.getInstance(ClaimService.class);
    claimService.setIsClaimFound(false);

  }

  public void handleException(String pdiNumber, Throwable e) throws ActionErrorException
  {
      log.error("Could not obtain record for PDI: " + pdiNumber, e);
      
      StringWriter sw = new StringWriter();
      PrintWriter p = new PrintWriter(sw);
      
      e.printStackTrace(p);
      
      String error = sw.toString();
      
      String lastError = "PDI: " + pdiNumber + " Stacktrace Follows: \n\n"
                     + error.replaceAll("\\)", "\\)\n")
                     + "-End of Error stacktrace-";
      
      invalidate();
      
      claimService.setLastError(lastError);
      
      FacesContext context = FacesContext.getCurrentInstance();
      context.addMessage("ewvApplication",
              new FacesMessage(FacesMessage.SEVERITY_ERROR, "Exception Stack Trace", lastError));
      
      // this will be picked up by page navigation. (See pages.xml)
      throw new ActionErrorException(lastError,e);
  }

  public void considerTestErrorAndDebug(String inPdi)
  throws IOException {
    // Check the properties file for notice that we should throw a test error
    java.util.Properties props = getProperties("EWV2Application.properties");
    if (props != null && !props.isEmpty()) {
      // Turn on debug if specified.  This only controls a single field near the Build message.
      Object doDebugObject = props.get("allowDebug");
      if (doDebugObject != null) {
        String doDebugString = (String) doDebugObject;
        Boolean doDebugYN = Boolean.valueOf(doDebugString);
        if (doDebugYN) {
          doDebug = true;
        }
      }
      log.info("Debug specified?" + doDebug);
      
      // Throw an exception if specified, for testing purposes.
      Object sampleErrorObject = props.get("showSampleError");
       if (sampleErrorObject != null) {
         String sampleErrorString = (String) sampleErrorObject;
         Boolean sampleErrorYN = Boolean.valueOf(sampleErrorString);
         if (sampleErrorYN) {
           if (inPdi.equals("888888888888888")) {
             log.info("Producing a test error");
             ClaimType claimType = null;
             claimType.getValueOf();
           }
         }
       }
    }
    
    //log.info("Not producing a test error");
  }

  final public java.util.Properties getProperties(final String pFileName)
  throws IOException {
    // If the incoming parameter is null, throw an IOException rather than
    // waiting for a NullPointerException later
    if (pFileName == null) {
      throw new IOException("Properties file name may not be null");
    }
    
    // Put an input stream onto the specified properties file
    InputStream inputStream = this.getClass().getClassLoader().getResourceAsStream(pFileName);
    
    // Load the file into a properties object for return
    final java.util.Properties props = new java.util.Properties();
    if (inputStream != null) {
      props.load(inputStream);
    }
    
    return props;
  }

}
